import { Button } from "@chakra-ui/button"
import { useDisclosure } from "@chakra-ui/react-use-disclosure"
import { Box, Container } from "@chakra-ui/layout"
import {
  Drawer,
  DrawerBody,
  DrawerCloseButton,
  DrawerContent,
  DrawerFooter,
  DrawerHeader,
  DrawerOverlay,
} from "@chakra-ui/modal"
import { chakra } from "@chakra-ui/system"
import * as React from "react"
import { ChangeEvent } from "react"
import {
  Accordion,
  AccordionButton,
  AccordionIcon,
  AccordionItem,
  AccordionPanel,
} from "../src"

export default {
  title: "Components / Disclosure / Accordion",
  decorators: [(story: Function) => <Container>{story()}</Container>],
}

/**
 * By default, only one accordion can be visible
 * at a time, and it can't be toggled.
 *
 * Note 🚨: Each accordion button must be wrapped in a heading tag,
 * that is appropriate for the information architecture of the page.
 */
export const Basic = () => (
  <Accordion>
    <AccordionItem>
      <h2>
        <AccordionButton>
          <chakra.div flex="1" textAlign="left">
            Section 1 title
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel>Panel 1</AccordionPanel>
    </AccordionItem>

    <AccordionItem>
      <h2>
        <AccordionButton>
          <chakra.div flex="1" textAlign="left">
            Section 2 title
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel>Panel 2</AccordionPanel>
    </AccordionItem>
  </Accordion>
)

export const allowToggle = () => (
  <Accordion allowToggle>
    <AccordionItem>
      <h2>
        <AccordionButton>
          <chakra.div flex="1" textAlign="left">
            Section 1 title
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel pb={4}>
        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod
        tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim
        veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
        commodo consequat.
      </AccordionPanel>
    </AccordionItem>

    <AccordionItem>
      <h2>
        <AccordionButton>
          <chakra.div flex="1" textAlign="left">
            Section 2 title
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel pb={4}>
        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod
        tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim
        veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
        commodo consequat.
      </AccordionPanel>
    </AccordionItem>
  </Accordion>
)

export const allowMultiple = () => (
  <Accordion allowMultiple>
    <AccordionItem>
      <h2>
        <AccordionButton>
          <chakra.div flex="1" textAlign="left">
            Section 1 title
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel pb={4}>
        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod
        tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim
        veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
        commodo consequat.
      </AccordionPanel>
    </AccordionItem>

    <AccordionItem>
      <h2>
        <AccordionButton>
          <chakra.div flex="1" textAlign="left">
            Section 2 title
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel pb={4}>
        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod
        tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim
        veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
        commodo consequat.
      </AccordionPanel>
    </AccordionItem>
  </Accordion>
)

export const stylingExpanded = () => (
  <Accordion allowToggle>
    <AccordionItem>
      <h2>
        <AccordionButton _expanded={{ bg: "tomato", color: "white" }}>
          <chakra.div flex="1" textAlign="left">
            Click me to see a different style
          </chakra.div>
          <AccordionIcon />
        </AccordionButton>
      </h2>
      <AccordionPanel>
        Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod
        tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim
        veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea
        commodo consequat.
      </AccordionPanel>
    </AccordionItem>
  </Accordion>
)

const data = [
  { title: "First Item", text: "Some value 1..." },
  { title: "Second Item", text: "Some value 2..." },
  { title: "Third Item", text: "Some value 3..." },
  { title: "Fourth Item", text: "Some value 4..." },
  { title: "Fifth Item", text: "Some value 5..." },
  { title: "Some other text", text: "Some value 6..." },
  { title: "Another one", text: "Some value 7..." },
]

export function Bug_2160() {
  const inputRef = React.useRef<HTMLInputElement>(null)
  const [displayData, setDisplayData] = React.useState(data)
  const [filter, setFilter] = React.useState("")

  React.useEffect(() => {
    if (!filter || filter === "") {
      setDisplayData(data)
    }

    const filteredData = data.filter((item) =>
      item.title.toLowerCase().includes(filter.toLowerCase()),
    )
    setDisplayData(filteredData)
  }, [filter])

  React.useEffect(() => {
    inputRef.current?.focus()
  }, [displayData])

  function onInputChange(e: ChangeEvent<HTMLInputElement>) {
    setFilter(e.target.value)
  }

  return (
    <chakra.div padding={4}>
      <chakra.div mt={3} mb={12}>
        <chakra.input
          ref={inputRef}
          value={filter}
          onChange={onInputChange}
          placeholder="Write filter for data title"
        />
      </chakra.div>
      {displayData.length > 0 && (
        <Accordion allowToggle>
          {displayData.map((item, i) => (
            <AccordionItem key={`accordion-item-${i}`}>
              <h2>
                <AccordionButton>
                  <chakra.div flex="1" textAlign="left">
                    {item.title}
                  </chakra.div>
                  <AccordionIcon />
                </AccordionButton>
              </h2>
              <AccordionPanel pb={4}>{item.text}</AccordionPanel>
            </AccordionItem>
          ))}
        </Accordion>
      )}
    </chakra.div>
  )
}

export const FocusBug = () => {
  const { isOpen, onOpen, onClose } = useDisclosure()

  return (
    <Box textAlign="center" fontSize="xl">
      <Button colorScheme="teal" onClick={onOpen}>
        Open
      </Button>
      <Drawer isOpen={isOpen} placement="right" onClose={onClose}>
        <DrawerOverlay />
        <DrawerContent>
          <DrawerCloseButton />
          <DrawerHeader>Create your account</DrawerHeader>

          <DrawerBody>
            <Accordion allowMultiple>
              <AccordionItem>
                <h2>
                  <AccordionButton>
                    <Box flex="1" textAlign="left">
                      Section 1 title
                    </Box>
                    <AccordionIcon />
                  </AccordionButton>
                </h2>
                <AccordionPanel pb={4}>
                  <a href="https://chakra-ui.com/should-not-have-focus-if-panel-closed-1">
                    Chakra 1
                  </a>
                  <a href="https://chakra-ui.com/should-not-have-focus-if-panel-closed-2">
                    Chakra 2
                  </a>
                  <a href="https://chakra-ui.com/should-not-have-focus-if-panel-closed-3">
                    Chakra 3
                  </a>
                </AccordionPanel>
              </AccordionItem>

              <AccordionItem>
                <h2>
                  <AccordionButton>
                    <Box flex="1" textAlign="left">
                      Section 2 title
                    </Box>
                    <AccordionIcon />
                  </AccordionButton>
                </h2>
                <AccordionPanel pb={4}>
                  <a href="https://chakra-ui.com/should-not-have-focus-if-panel-closed-4">
                    Chakra 1
                  </a>
                  <a href="https://chakra-ui.com/should-not-have-focus-if-panel-closed-5">
                    Chakra 2
                  </a>
                  <a href="https://chakra-ui.com/should-not-have-focus-if-panel-closed-6">
                    Chakra 3
                  </a>
                </AccordionPanel>
              </AccordionItem>
            </Accordion>
          </DrawerBody>

          <DrawerFooter>
            <Button variant="outline" mr={3} onClick={onClose}>
              Cancel
            </Button>
            <Button color="blue">Save</Button>
          </DrawerFooter>
        </DrawerContent>
      </Drawer>
    </Box>
  )
}

export const WithDisabledAccordionItem = () => {
  return (
    <Accordion index={1}>
      <AccordionItem isDisabled>
        <AccordionButton>Button 1</AccordionButton>
        <AccordionPanel>One Content</AccordionPanel>
      </AccordionItem>
      <AccordionItem isDisabled>
        <AccordionButton>Button 2</AccordionButton>
        <AccordionPanel>Two Content</AccordionPanel>
      </AccordionItem>
      <AccordionItem>
        <AccordionButton>Button 3</AccordionButton>
        <AccordionPanel>Three Content</AccordionPanel>
      </AccordionItem>
      <AccordionItem isDisabled>
        <AccordionButton>Button 4</AccordionButton>
        <AccordionPanel>Four Content</AccordionPanel>
      </AccordionItem>
      <AccordionItem>
        <AccordionButton>Button 5</AccordionButton>
        <AccordionPanel>Five Content</AccordionPanel>
      </AccordionItem>
    </Accordion>
  )
}

// export const OverridableAccordionContainer = () => {
//   return (
//     <ChakraProvider
//       theme={extendTheme({
//         components: {
//           Accordion: {
//             baseStyle: {
//               root: {
//                 backgroundColor: "red",
//               },
//               container: {
//                 backgroundColor: "green",
//               },
//             },
//           },
//         },
//       })}
//     >
//       <Accordion allowToggle>
//         <AccordionItem>
//           <AccordionButton>This should be green</AccordionButton>
//           <AccordionPanel>Turns out it's red</AccordionPanel>
//         </AccordionItem>
//       </Accordion>
//     </ChakraProvider>
//   )
// }
